/*
 * RadixDocument.java
 *
 * Created on 07 July 2006, 18:30
 *
 * Copyright  2006 Miles Asvachin
 * Feel free to do whatever you want with this code, but please provide credit
 * where appropriate.
 */

package radixreducer;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Allows only a subset of characters 0-9 and A-Z to be entered,
 * suitable for bases up to 36, plus a negative sign and a radix point.
 *
 * @version 1.1 2 October 2006
 * @author Miles Asvachin
 */
public class RadixDocument extends PlainDocument
{
	private final char[] validChars;
	
	/**
	 * Creates a RadixDocument that will accept characters appropriate for the given base.
	 *
	 * @param base the base the document will restrict input to
	 */
	public RadixDocument(int base){validChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".substring(0, base).toCharArray();}
	
	public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
	{
		if(offs == 0 && getText(0, 1).startsWith("-"))
			return;
		
		char[] source = str.toUpperCase().toCharArray();
		char[] result = new char[source.length];
		int j = 0;
		boolean hasRadixPoint = getText(0, getLength()).contains(".");
		
		for(int i = 0; i < source.length; i++)
		{
			//Allow a negative sign at the beginning.
			if(i == 0 && source[0] == '-' && offs == 0 && !"-".equals(getText(0, 1)))
			{
				result[j++] = source[i];
				continue;
			}
			
			//Any valid digit.
			if(isValidChar(source[i]))
			{
				result[j++] = source[i];
				continue;
			}
			
			//One and only one radix point.
			if(source[i] == '.' && !hasRadixPoint)
			{
				result[j++] = source[i];
				hasRadixPoint = true;
				continue;
			}
		}
		
		super.insertString(offs, new String(result, 0, j), a);
	}
	
	private boolean isValidChar(char c){return java.util.Arrays.binarySearch(validChars, c) >= 0;}
}